﻿using System;
using System.IO;
using System.Windows.Forms;
using GitCommands;
using GitCommands.Repository;
using ResourceManager.Translation;

namespace GitUI
{
    public partial class Open : GitExtensionsForm
    {
        private readonly TranslationString _warningOpenFailed =
            new TranslationString("Directory does not exist.");

        private readonly TranslationString _warningOpenFailedCaption =
            new TranslationString("Error");
        private GitModule choosenModule = null;

        public Open()
        {
            InitializeComponent();
            Translate();

            _NO_TRANSLATE_Directory.DataSource = Repositories.RepositoryHistory.Repositories;
            _NO_TRANSLATE_Directory.DisplayMember = "Path";

            Load.Select();

            _NO_TRANSLATE_Directory.Focus();
            _NO_TRANSLATE_Directory.Select();
        }

        public static GitModule OpenModule(IWin32Window owner)
        {
            using (var open = new Open())
            {
                open.ShowDialog(owner);
                return open.choosenModule;
            }

        }

        private void BrowseClick(object sender, EventArgs e)
        {
            using (var browseDialog = new FolderBrowserDialog { SelectedPath = _NO_TRANSLATE_Directory.Text })
            {

                if (browseDialog.ShowDialog(this) == DialogResult.OK)
                {
                    _NO_TRANSLATE_Directory.Text = browseDialog.SelectedPath;
                }
            }
        }

        private void LoadClick(object sender, EventArgs e)
        {
            if (Directory.Exists(_NO_TRANSLATE_Directory.Text))
            {
                choosenModule = new GitModule(_NO_TRANSLATE_Directory.Text);

                Repositories.AddMostRecentRepository(choosenModule.WorkingDir);
            }
            else
            {
                DialogResult = DialogResult.None;
                MessageBox.Show(this, _warningOpenFailed.Text, _warningOpenFailedCaption.Text);
            }
        }

        private void Open_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (DialogResult == DialogResult.None)
                e.Cancel = true;
        }

        private void DirectoryKeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char) Keys.Enter)
            {
                LoadClick(null, null);
            }
        }
    }
}